function [mu_hat , Theta1] = firstOrderFit(y , Theta0)
%
% function [mu_hat , Theta1] = firstOrderFit(y , Theta0)
%
% Compute 1st order polynomial fits of feature values y.
% Features have normally distributed conditional expectation, and expectation which is
% linearly time-dependend; i.e.
%        y(x) = m * x + q + \eta \eta ~ N(0 , \sigma^2)
% where t corresponds to y indexes i.e. x = [0 : 1 : length(y) - 1];
%
% The function works in two modes: training and validation
% during training only the values of the features are given as input to the function with the reference point
% while in the validation
%
%
% if isstruct(Theta0) == 0 %% Training
%
% INPUT
%
% y                 sequence of feature values
% point = Theta0    point where the regression line has to be evaluted
%
%
% if isstruct(Theta0) == 1  %% Validation
%
%
% INPUT
%
% y                 sequence of feature values
% Theta0             structure containing the feature-related variables to avoid to compute fitting out of scratches
%
%
% OUTPUT
%
% mu_hat            \hat{y}(Theta0.point) = \hat{m} * Theta0.point + \hat{q}
% Theta1            updated structure, conatinig the following fields
%   Theta1.n                 number of considered observations
%   Theta1.Sx                \sum_{i = 1}^{n} x(i)
%   Theta1.Sxx               \sum_{i = 1}^{n} (x(i).^2)
%   Theta1.Sy                \sum_{i = 1}^{n} y(i)
%   Theta1.Sxy               \sum_{i = 1}^{n} (x(i) * y(i))
%   Theta1.m_hat             estimate of m
%   Theta1.q_hat             estimate of q
%   Theta1.sigma_noise       \sigma meant as std[y(t)|t]
%   Theta1.point             reference point where to compute the polynomial fit
%   Theta1.mu_hat            \hat{y}(Theta0.point) = \hat{m} * Theta0.point + \hat{q}
%   Theta1.stdEst            standard deviation of the estimator Theta1.mu_hat
%
% Giacomo Boracchi
% Politecnico di Milano
% December 2010
% giacomo.boracchi@polimi.it
%
% Revision History
%

if isstruct(Theta0) == 0
    do_training = 1;
    point = Theta0;
else
    do_training = 0;
    point = Theta0.point;
end

% training
if do_training == 1
    n = length(y);
    x = [0 : n - 1];
    Sx = sum(x);
    Sxx = sum(x.^2);
    Sy = sum(y);
    Sxy = sum(x.*y);
    
    % compute the updated estimates
    m_hat = (Sxy - Sx * Sy/n) / (Sxx - (Sx^2)/n);
    q_hat = Sy/n - m_hat * Sx/n;
    
    y_hat = m_hat .* x + q_hat;
    
    % compute std of eta , i.e. sigma
    sigma_noise = sqrt(sum((y - y_hat).^2)/(n-2));
    theta.sigma_noise  = sigma_noise;
    % TODO. FIXME add a robust estimator on detrended data
    
else
    % validation
    
    % assign the previous estimate
    Theta1 = Theta0;
    
    % update the first element of the structure array, trends on other features are not considered 
    theta = Theta0(1);
    sigma_noise = theta.sigma_noise;
    n = theta.n;
    
    x_curr = n;
    Sx = theta.Sx + x_curr; % cumulativa su x
    Sxx = theta.Sxx +  x_curr^2; % cumulativa su x^2
    Sy = theta.Sy + y(end); % cumulativa su y
    Sxy = theta.Sxy + x_curr * y(end); % cumulativa su xy
    n = n + 1;
    
    % compute the updated estimates
    m_hat = (Sxy - Sx * Sy/n) / (Sxx - (Sx^2)/n);
    q_hat = Sy/n - m_hat * Sx/n;
    
end

% compute the estimate
mu_hat = m_hat .* point + q_hat;

% compute the variance of the corresponding estimator
var_m_hat = (sigma_noise ^ 2) / (Sxx - (Sx^2)/n);
var_q_hat = (sigma_noise ^ 2) * ( 1 / n + (Sx / n) ^ 2 / (Sxx - (Sx^2)/n));  
%     disp([' sigma noise = ' num2str(sigma_noise) ' 1/n = ' num2str(1/n) ' denominator = '  num2str((Sx / n) ^ 2 / (Sxx -
%     (Sx^2)/n))])
cov_m_q_hat = - (sigma_noise ^ 2) * (Sx/n) / (Sxx - (Sx^2)/n);
% var(aX +bY) = a^2 var(X) + b^2 var(Y) + 2ab cov(X,Y)
stdEst = sqrt(var_m_hat * (point)^2 + var_q_hat + 2 * (point) * var_m_hat * var_q_hat);

% update output fields
theta.n = n;
theta.Sx = Sx;
theta.Sxx = Sxx;
theta.Sy = Sy;
theta.Sxy = Sxy;
theta.m_hat = m_hat;
theta.q_hat = q_hat;
theta.mu_hat = mu_hat;
theta.stdEst = stdEst;

Theta1(1) = theta;

if do_training == 1
    Theta1.point = point;
end